﻿using UnityEngine;
using System.Collections;

/*Script used to control our player such as jump, run, walk, duck, etc*/
/*These actions will be based on keys read in from the keyboard or inputs from the 9-axis sensor*/

public class PlayerController : MonoBehaviour
{
    //CONSTANTS
    const float INITIAL_Z = -10f;
    const float INITIAL_Y = 5.38f;
    const float INITIAL_X = 9.80f;
    //const float INITIAL_X = 0f;
    const float OUTOFBOUNDS_Y = -2.45f;
    const float FLAGPOLE = 211.68f;
    const float ENDOFLEVELBOUNDS = 218.27f;
    int NUMBEROFCOINBLOCKS = 15;
    const int DEFAULT = 48;
    const int JUMP = 49;
    const int DUCK = 2;
    const int LEFT =51;
    const int RIGHT =52;
    const int FASTRIGHT = 53;
    const int FASTLEFT = 54;
    const int PAUSE = 55;
    const int DOWN = 56;
   
    //variables used to check various things such as on the ground or hitting a block
    public float groundCheckRadius;
    public Transform groundCheck;
    public LayerMask whatIsGround;
    public LayerMask whatIsSmashable;
    public LayerMask whatIsSmashableTurtle;
    public LayerMask whatIsPipe;


     //varibles to controller our player
    public float moveSpeed;
    float moveVelocity;
    public float jumpHeight;
    private Vector3 initialPosition;
    private Vector3 initialCamera;

    //gameobjects
    GameObject mainCamera;
    GameObject currentObject;
    GameObject pipeMiddle1;
    GameObject pipeTop1;
    private Animator animator;

    //int[] numbers; // declare numbers as an int array of any size
    //numbers = new int[10];  // numbers is a 10-element array
    //numbers = new int[20];  // now it's a 20-element array
    GameObject[] coinBlocks;
    Animator[] coinBlockAnimators;
    bool[] hasHitCoinBlocks;


    //other vars
    public Vector3 cameraLocation;

    //bools
    private bool grounded = true;
    public static bool isSmashing = false;
    public static bool isSmashingTurtle = false;
    public static bool isOverPipe = false;
    public static bool isDying = false;
    public static bool isPaused = false;
    public static bool isGameOver = false;
    public static bool isBouncing = false;
    private bool levelIsFinished = false;

    private static bool isUnderGround = false;

    void Awake()
    {
        if (isUnderGround)
        {
            transform.position = new Vector3(174.94f, 3.188f);
        }
        DontDestroyOnLoad(this.gameObject);
        
    }

	void Start ()
    {
        //initialize all of our coin blocks from the hierarchy
        //we need the gameobject itself, the animators, and booleans
        coinBlocks = new GameObject[NUMBEROFCOINBLOCKS];
        coinBlockAnimators = new Animator[NUMBEROFCOINBLOCKS];
        hasHitCoinBlocks = new bool[NUMBEROFCOINBLOCKS];

        for (int i = 0; i < NUMBEROFCOINBLOCKS; i++)
        {
            coinBlocks[i] = GameObject.Find("EmptyCoinBlock"+i);
            coinBlockAnimators[i] = coinBlocks[i].GetComponent<Animator>();
            hasHitCoinBlocks[i] = false;
        }

        //refer to various gameobject in the hierarchy
        mainCamera = GameObject.Find("Main Camera");
        pipeMiddle1 = GameObject.Find("PipeMiddle1");
        pipeTop1 = GameObject.Find("PipeTop1");

        //get components already on the player
        animator = GetComponent<Animator>();

        //get the initial position of our player
        //initialPosition = GetComponent<Transform>().position;

        //get camera to initial position
        //initialCamera = mainCamera.GetComponent<Transform>().position;
       
	}//end start

/*gets called at a fixed amount of time*/
    void FixedUpdate()
    {
        //use a small circle to detect if we are on an object in the foreground that we can jump off of
        grounded = Physics2D.OverlapCircle(groundCheck.position, groundCheckRadius, whatIsGround);
        isSmashing = Physics2D.OverlapCircle(groundCheck.position, groundCheckRadius, whatIsSmashable);
        isSmashingTurtle = Physics2D.OverlapCircle(groundCheck.position, groundCheckRadius, whatIsSmashableTurtle);
        isOverPipe = Physics2D.OverlapCircle(groundCheck.position, groundCheckRadius, whatIsPipe);

        if (isSmashing || isSmashingTurtle)
        {
            isBouncing = true;
        }
    }
	
	void Update ()
    {



        //disable these actions if the game is paused
        if (!isPaused && !isDying)
        {
            PlayerJump();
            PlayerMove();
            FollowPlayer();
            DownPipe();
            CompleteLevel();
        }

        PlayerDie();

        //check to see if the game is paused or if it is game over
        PauseGame();
        GameOver();
	}//end update


/*Method controlling how player jumps*/
    void PlayerJump()
    {
        //allows player to jump a selected height and retain current x velocity as long as we are on the ground
        if ((Input.GetKeyDown(KeyCode.UpArrow) || SerialCommunication.accValue == JUMP) || isBouncing || Input.GetKeyDown(KeyCode.Joystick1Button1) )
        {
            if (grounded || isBouncing)
            {
                GetComponent<Rigidbody2D>().velocity = new Vector2(GetComponent<Rigidbody2D>().velocity.x, jumpHeight); 
                SoundEffects.jump.Play();
            }
        }

        animator.SetBool("Grounded", grounded);
        isBouncing = false;

    }//end PlayerJump

    void PlayerMove()
    {
        //player is standing still by default
        moveVelocity = 0;

        //allows player to move left and right while retaining values in the Y direction
        if (Input.GetKey(KeyCode.D) || (SerialCommunication.accValue == RIGHT || SerialCommunication.accValue == FASTRIGHT) || Input.GetAxis("LeftHorizontal") < 0 )
        {
            if (Input.GetKey(KeyCode.S) || SerialCommunication.accValue == FASTRIGHT || Input.GetKey(KeyCode.Joystick1Button2))
            { 
                moveVelocity = 2f * moveSpeed;
            }
            else
            {
                moveVelocity = moveSpeed;
            }
        }

        if (Input.GetKey(KeyCode.A) || ( SerialCommunication.accValue == LEFT || SerialCommunication.accValue == FASTLEFT ) || Input.GetAxis("LeftHorizontal") > 0 )
        {
            if (Input.GetKey(KeyCode.S) || SerialCommunication.accValue == FASTLEFT || Input.GetKey(KeyCode.Joystick1Button2))
            {
                moveVelocity = -1f * 2f * moveSpeed;
            }
            else
            {
                moveVelocity = -1f * moveSpeed;
            }
        }

        //update player's speed
        GetComponent<Rigidbody2D> ().velocity = new Vector2 (moveVelocity, GetComponent<Rigidbody2D> ().velocity.y);
        //refer to the speed variable found in the animator and assign our current x velocity to the animation speed
        animator.SetFloat("Speed", Mathf.Abs(GetComponent<Rigidbody2D>().velocity.x) );

        //depending on which way the player is running face him in the appropriate direction
        //this avoids having to create a seperate animation
        if (GetComponent<Rigidbody2D>().velocity.x > 0)
        {
            transform.localScale = new Vector3(1f, 1f, 1f);
        }
        else if (GetComponent<Rigidbody2D>().velocity.x < 0)
        {
            transform.localScale = new Vector3(-1f, 1f, 1f);
        }
        
    }//end playerMove
        

    void FollowPlayer()
    {
        //start camera scrolling once we are near the middle of the screen
        if (transform.position.x > INITIAL_X)
        {
            //update camera location so that it follows the player (only in the x direction)
            cameraLocation = (new Vector3(transform.position.x, INITIAL_Y , INITIAL_Z));
            mainCamera.transform.position = cameraLocation;
        }
    }//end FollowPlayer


    void OnTriggerEnter2D(Collider2D objectCollider)
    {
        for (int i = 0; i < NUMBEROFCOINBLOCKS; i++)
        {
            if (!hasHitCoinBlocks[i] && objectCollider.name == coinBlocks[i].name)
            {
                SoundEffects.coin.Play();
                hasHitCoinBlocks[i] = true;
                coinBlockAnimators[i].SetBool("hasHitCoinBlock", hasHitCoinBlocks[i]);
                GameGUI.numberOfCoins++;
            }//end if


            if (objectCollider.name == "PipeMiddleG")
            {
                SoundEffects.pipe.Play();
                GetComponent<Rigidbody2D>().constraints = RigidbodyConstraints2D.FreezePositionY;
                objectCollider.GetComponent<BoxCollider2D>().enabled = false;
                GetComponent<SpriteRenderer>().sortingOrder = 0;
                InvokeRepeating("LoadAboveGround",1f,1f);
            }
        }//end for





    }//end on trigger
        
/*Check to see if the player has died*/

    void PlayerDie()
    {
        if ( (GetComponent<Transform>().position.y <= OUTOFBOUNDS_Y || GoombaController.hasHitMario ||TurtleController.hasHitMario ) && !isDying && !isGameOver)
        {
            SoundEffects.theme1.Stop();
            SoundEffects.die.Play();
            GoombaController.hasHitMario = false;
            TurtleController.hasHitMario = false;
            isDying = true;
            animator.SetBool("isDying", isDying);
            GetComponent<Rigidbody2D>().constraints = RigidbodyConstraints2D.FreezeAll;
            GameGUI.numberOfLives--;

            if (GameGUI.numberOfLives <= 0)
            {
                isGameOver = true;
            }
        }

        if (!SoundEffects.die.isPlaying && isDying)
        {
            isDying = false;
            if (!isGameOver)
            {
                //reset the screen because we died
          
                UnityEngine.SceneManagement.SceneManager.LoadScene(UnityEngine.SceneManagement.SceneManager.GetActiveScene().buildIndex);  
            }
            SoundEffects.gameOver.Play();
        }
    }

/*Method used to toggle the isPaused flag*/
//if our game is paused then movement is disabled and pause it written on the gui
    void PauseGame()
    {

        if (Input.GetKeyDown(KeyCode.P) || SerialCommunication.accValue == PAUSE )
        {
            if (Input.GetKeyDown(KeyCode.P) || SerialCommunication.previousAccValue == DEFAULT)
            {
                isPaused = !isPaused;
                SoundEffects.pause.Play();

                if (isPaused)
                {
                    GetComponent<Rigidbody2D>().constraints = RigidbodyConstraints2D.FreezeAll;
                }
                else
                {
                    GetComponent<Rigidbody2D>().constraints = RigidbodyConstraints2D.None;
                    GetComponent<Rigidbody2D>().constraints = RigidbodyConstraints2D.FreezeRotation;
                }
            }
            //Time.timeScale to 0.0 
        }
    }//end pause game
        

    void GameOver()
    {
        //if gameover and no longer dying
        if (isGameOver && !isDying)
        {
            if (!SoundEffects.gameOver.isPlaying)
            {
                GameGUI.numberOfLives = 3;
                GameGUI.numberOfCoins = 0;
                UnityEngine.SceneManagement.SceneManager.LoadScene(UnityEngine.SceneManagement.SceneManager.GetActiveScene().buildIndex);
            }

        }
    }//end GameOver

    void DownPipe()
    {
        if ( (Input.GetKeyDown(KeyCode.DownArrow) || SerialCommunication.accValue == DOWN ) && isOverPipe )
        {
            //play pipe sound, freeze movement, shift background
            SoundEffects.pipe.Play();
            GetComponent<Rigidbody2D>().constraints = RigidbodyConstraints2D.FreezePositionX;
            pipeMiddle1.GetComponent<BoxCollider2D>().enabled = false;
            pipeMiddle1.GetComponent<SpriteRenderer>().sortingOrder = 6;
            pipeTop1.GetComponent<SpriteRenderer>().sortingOrder = 7;

            //once the pipe sound if played change scenes
            InvokeRepeating("LoadUnderGround",1f,1f);

        }
    }//DownPipe


/*used to load underground*/
    void LoadUnderGround()
    {
        UnityEngine.SceneManagement.SceneManager.LoadScene("UnderGround 1-1");
        CancelInvoke();
    }

/*used to Load above ground*/
    void LoadAboveGround()
    {
        isUnderGround = true;
        UnityEngine.SceneManagement.SceneManager.LoadScene("Level 1-1");
        CancelInvoke();
    }

/*checks to see if we have completed the level*/
    void CompleteLevel()
    {
        //if the player reaches the end of the level play the ending music and then exit application
        if (GetComponent<Transform>().position.x >= FLAGPOLE && !levelIsFinished)
        {
            levelIsFinished = true;
            SoundEffects.theme1.Stop();
            SoundEffects.levelClear.Play();

        }

        if (levelIsFinished && !SoundEffects.levelClear.isPlaying)
        {
            
            UnityEditor.EditorApplication.isPlaying = false;
        }
    }

}//end script
